Package: av
Type: Package
Title: Working with Audio and Video in R
Version: 0.8.5
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroen@berkeley.edu",
    comment = c(ORCID = "0000-0002-4035-0289"))
Description: Bindings to 'FFmpeg' <http://www.ffmpeg.org/> AV library for working with 
    audio and video in R. Generates high quality video from images or R graphics with 
    custom audio. Also offers high performance tools for reading raw audio, creating
    'spectrograms', and converting between countless audio / video formats. This package 
    interfaces directly to the C API and does not require any command line utilities.
License: MIT + file LICENSE
URL: https://ropensci.r-universe.dev/av, https://docs.ropensci.org/av/
BugReports: https://github.com/ropensci/av/issues
Encoding: UTF-8
RoxygenNote: 7.2.1
SystemRequirements: FFmpeg (>= 3.2); with at least libx264 and lame
        (mp3) drivers. MacOS Homebrew: ffmpeg. Debian/Ubuntu:
        libavfilter-dev. Fedora/CentOS: either ffmpeg-free-devel or
        (via https://rpmfusion.org) ffmpeg-devel.
Depends: R (>= 3.5)
Imports: graphics
Config/pkgdown: seewave, ggplot2, phonTools, signal, tuneR
Suggests: testthat, ps, ggplot2, gapminder
Language: en-US
NeedsCompilation: yes
Packaged: 2023-09-25 17:12:38 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (<https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroen@berkeley.edu>
Repository: CRAN
Date/Publication: 2023-09-25 20:40:02 UTC

Package: av
Type: Package
Title: Working with Audio and Video in R
Version: 0.9.6
Authors@R: 
    person("Jeroen", "Ooms", , "jeroenooms@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4035-0289"))
Description: Bindings to 'FFmpeg' <http://www.ffmpeg.org/> AV library for working with 
    audio and video in R. Generates high quality video from images or R graphics with 
    custom audio. Also offers high performance tools for reading raw audio, creating
    'spectrograms', and converting between countless audio / video formats. This package 
    interfaces directly to the C API and does not require any command line utilities.
License: MIT + file LICENSE
URL: https://ropensci.r-universe.dev/av, https://docs.ropensci.org/av/
BugReports: https://github.com/ropensci/av/issues
Encoding: UTF-8
RoxygenNote: 7.3.2
SystemRequirements: FFmpeg (>= 3.2); with at least libx264 and lame
        (mp3) drivers. MacOS Homebrew: ffmpeg. Debian/Ubuntu:
        libavfilter-dev. Fedora/RHEL: either ffmpeg-devel from
        https://rpmfusion.org (preferred), or libavfilter-free-devel
        which is a more limited version available on Fedora/EPEL9.
Depends: R (>= 3.5)
Imports: graphics
Config/Needs/website: seewave, ggplot2, phonTools, signal, tuneR
Suggests: curl, testthat, ps, ggplot2, gapminder
Language: en-US
NeedsCompilation: yes
Packaged: 2025-10-19 16:45:30 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (ORCID: <https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-20 05:10:40 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,av_fft)
export(av_audio_convert)
export(av_capture_graphics)
export(av_decoders)
export(av_demo)
export(av_demuxers)
export(av_encode_video)
export(av_encoders)
export(av_filters)
export(av_log_level)
export(av_media_info)
export(av_muxers)
export(av_spectrogram_video)
export(av_video_convert)
export(av_video_images)
export(av_video_info)
export(bartlett)
export(bhann)
export(bharris)
export(blackman)
export(bnuttall)
export(bohman)
export(cauchy)
export(dolph)
export(flattop)
export(gauss)
export(hamming)
export(hanning)
export(lanczos)
export(nuttall)
export(parzen)
export(read_audio_bin)
export(read_audio_fft)
export(sine)
export(tukey)
export(welch)
export(write_audio_bin)
importFrom(graphics,abline)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,par)
useDynLib(av,R_audio_bin)
useDynLib(av,R_audio_fft)
useDynLib(av,R_convert_audio)
useDynLib(av,R_encode_video)
useDynLib(av,R_generate_window)
useDynLib(av,R_get_open_handles)
useDynLib(av,R_list_codecs)
useDynLib(av,R_list_demuxers)
useDynLib(av,R_list_filters)
useDynLib(av,R_list_muxers)
useDynLib(av,R_log_level)
useDynLib(av,R_video_info)

# AV-98

AV-98 is a command line client for the Gemini protocol, written in Python and
released under the BSD 2-Clause License.  It features a unique and minimalistic
user interface designed to facilitate rapid keyboard-driven navigation of
Geminispace.  The code for the VF-1 Gopher client was used as a basis to
develop AV-98 and the user interface remains very similar.  Users of one will
feel very comfortable with the other.

AV-98 naturally has
[its own Gemini
capsule](gemini://zaibatsu.circumlunar.space/~solderpunk/software/av98/).
You can use AV-98 (or any other Gemini client!) to access ASCII-formatted
versions of the AV-98 man page there, along with other useful information
on installation, usage, release history, etc.

The source code lives in a
[git repo hosted at Sourcehut](https://git.sr.ht/~solderpunk/AV-98).  There is
also [a ticket tracker](https://todo.sr.ht/~solderpunk/AV-98) you can use to
report bugs.

The repository contains the following files (plus some development / packaging
related noise):

* LICENSE - text of BSD 2-Clause License
* README.md - the README file you are currently reading
* docs/av98.1 - troff source for the av98(1) man page
* src/av98/ - Python source for the AV-98 module

AV-98 has no "strict dependencies", i.e. it will run and work without anything
else beyond the Python standard library.  However, it will "opportunistically
import" a few other libraries if they are available to offer an improved
experience.

* The [ansiwrap library](https://pypi.org/project/ansiwrap/) may result in
  neater display of text which makes use of ANSI escape codes to control colour.
* The [cryptography library](https://pypi.org/project/cryptography/) will
  provide a better and slightly more secure experience when using the default
  TOFU certificate validation mode and is highly recommended.

AV-98 was developed by Solderpunk with contributions many other helpful
Geminispace folk.  See the top of `src/av98/main.py` soure file for all
contributors.

# AV - API Versioning Macro

A Rust procedural macro for tracking API version history and deprecation warnings.

## Features

- Track multiple version entries per function/struct
- Auto-generate documentation from version history
- Auto-generate warnings based on version status

## Version Types

- **`unstable`** - Generates warning when `deprecated_for_unstable` feature is enabled (controlled by API developer)
- **`stable`** - No warnings
- **`update`** - No warnings
- **`deprecated`** - Always generates `#[deprecated]` warning

## Usage

```rust
use av::ver;

// Unstable API
#[ver(unstable, since = "0.1.0")]
pub fn unstable_example() { }

// Stable API
#[ver(stable, since = "1.0.0")]
pub fn stable_example() { }

// Deprecated API - always generates warning
#[ver(deprecated, since = "2.0.0", note = "Use new_function instead")]
pub fn old_function() { }

// Full - multiple versions with history
#[ver(
    update, since = "1.2.0", note = "Added parameter", date = "2024-03-01", author = "Akari";
    stable, since = "1.1.0", note = "First stable release", date = "2024-02-01", author = "Akari";
    unstable, since = "0.1.0", note = "Initial implementation", date = "2024-01-01", author = "Akari"
)]
pub fn full_example(value: i32, new_param: bool) { }
```

## Screenshots

**Deprecated Warning:**
![Deprecated Warning](static/deprecated.png)

**Generated Documentation:**
![Documentation](static/doc_gen.png)

#' Record Video from Graphics Device
#'
#' Runs the expression and captures all plots into a video. The [av_spectrogram_video]
#' function is a wrapper that plots data from [read_audio_fft] with a moving bar and
#' background audio.
#'
#' @export
#' @rdname capturing
#' @name capturing
#' @family av
#' @inheritParams encoding
#' @param expr an R expression that generates the graphics to capture
#' @param width width in pixels of the graphics device
#' @param height height in pixels of the graphics device
#' @param ... extra graphics parameters passed to [png()]
#' @examples
#' \donttest{
#' library(gapminder)
#' library(ggplot2)
#' makeplot <- function(){
#'   datalist <- split(gapminder, gapminder$year)
#'   lapply(datalist, function(data){
#'     p <- ggplot(data, aes(gdpPercap, lifeExp, size = pop, color = continent)) +
#'       scale_size("population", limits = range(gapminder$pop)) + geom_point() + ylim(20, 90) +
#'       scale_x_log10(limits = range(gapminder$gdpPercap)) + ggtitle(data$year) + theme_classic()
#'     print(p)
#'   })
#' }
#'
#' # Play 1 plot per sec, and use an interpolation filter to convert into 10 fps
#' video_file <- file.path(tempdir(), 'output.mp4')
#' av_capture_graphics(makeplot(), video_file, 1280, 720, res = 144, vfilter = 'framerate=fps=10')
#' av::av_media_info(video_file)
#' # utils::browseURL(video_file)}
av_capture_graphics <- function(expr, output = 'output.mp4', width = 720, height = 480, framerate = 1,
                       vfilter = "null", audio = NULL, verbose = TRUE, ...){
  imgdir <- tempfile('tmppng')
  dir.create(imgdir)
  on.exit(unlink(imgdir, recursive = TRUE))
  filename <- file.path(imgdir, "tmpimg_%05d.png")
  grDevices::png(filename, width = width, height = height, ...)
  graphics::par(ask = FALSE)
  tryCatch(eval(expr), finally = grDevices::dev.off())
  images <- list.files(imgdir, pattern = 'tmpimg_\\d{5}.png', full.names = TRUE)
  av_encode_video(images, output = output, framerate = framerate, vfilter = vfilter, audio = audio, verbose = verbose)
}

#' @export
#' @param audio path to media file with audio stream
#' @rdname capturing
av_spectrogram_video <- function(audio, output = "output.mp4", framerate = 25, verbose = TRUE, ...){
  fftdata <- read_audio_fft(audio)
  duration <- attr(fftdata, 'duration')
  movie <- av_capture_graphics({
    for(i in seq(0, duration, by = 1/framerate)){
      cat(sprintf("\rPlotting frame at %.2f sec...", i), file = stderr())
      graphics::plot(fftdata, vline = i)
    }
  }, framerate = framerate, audio = audio, verbose = verbose, output = output, ...)
}

library(testthat)
library(av)

if (ps::ps_is_supported()) {
  # This sometimes has false positives on MacOS https://github.com/r-lib/ps/issues/90
  reporter <- ps::CleanupReporter(testthat::CheckReporter)$new(proc_cleanup = FALSE, proc_fail = FALSE, conn_fail = FALSE)
  test_check("av", reporter = reporter)
} else {
  test_check("av")
}

#' Window functions
#'
#' Several common [windows function](https://en.wikipedia.org/wiki/Window_function)
#' generators. The functions return a vector of weights to use in [read_audio_fft].
#'
#' @export
#' @name window functions
#' @rdname winfunc
#' @param n size of the window (number of weights to generate)
#' @examples # Window functions
#' plot(hanning(1024), type = 'l', xlab = 'window', ylab = 'weight')
#' lines(hamming(1024), type = 'l', col = 'red')
#' lines(bartlett(1024), type = 'l', col = 'blue')
#' lines(welch(1024), type = 'l', col = 'purple')
#' lines(flattop(1024), type = 'l', col = 'darkgreen')
hanning <- function(n){
  generate_window(n, 1L)
}

#' @export
#' @rdname winfunc
hamming <- function(n){
  generate_window(n, 2L)
}

#' @export
#' @rdname winfunc
blackman <- function(n){
  generate_window(n, 3L)
}

#' @export
#' @rdname winfunc
bartlett <- function(n){
  generate_window(n, 4L)
}

#' @export
#' @rdname winfunc
welch <- function(n){
  generate_window(n, 5L)
}

#' @export
#' @rdname winfunc
flattop <- function(n){
  generate_window(n, 6L)
}

#' @export
#' @rdname winfunc
bharris <- function(n){
  generate_window(n, 7L)
}

#' @export
#' @rdname winfunc
bnuttall <- function(n){
  generate_window(n, 8L)
}

#' @export
#' @rdname winfunc
sine <- function(n){
  generate_window(n, 9L)
}

#' @export
#' @rdname winfunc
nuttall <- function(n){
  generate_window(n, 10L)
}

#' @export
#' @rdname winfunc
bhann <- function(n){
  generate_window(n, 11L)
}

#' @export
#' @rdname winfunc
lanczos <- function(n){
  generate_window(n, 12L)
}

#' @export
#' @rdname winfunc
gauss <- function(n){
  generate_window(n, 13L)
}

#' @export
#' @rdname winfunc
tukey <- function(n){
  generate_window(n, 14L)
}

#' @export
#' @rdname winfunc
dolph <- function(n){
  generate_window(n, 15L)
}

#' @export
#' @rdname winfunc
cauchy <- function(n){
  generate_window(n, 16L)
}

#' @export
#' @rdname winfunc
parzen <- function(n){
  generate_window(n, 17L)
}

#' @export
#' @rdname winfunc
bohman <- function(n){
  generate_window(n, 19L)
}

#' @useDynLib av R_generate_window
generate_window <- function(n, type){
  n <- as.integer(n)
  type <- as.integer(type)
  assert_range(n)
  assert_range(type, max = 19)
  .Call(R_generate_window, n, type)
}

assert_range <- function(x, min = 0, max = Inf){
  stopifnot(length(x) == 1)
  stopifnot(x >= min)
  stopifnot(x <= max)
}

#' Logging
#'
#' Get or set the [log level](https://www.ffmpeg.org/doxygen/4.0/group__lavu__log__constants.html).
#'
#' @useDynLib av R_log_level
#' @family av
#' @name logging
#' @export
#' @param set new [log level](https://www.ffmpeg.org/doxygen/4.0/group__lavu__log__constants.html) value
av_log_level <- function(set = NULL){
  if(length(set)){
    levels <- seq(-8, 56, 8)
    if(!(set %in% levels))
      stop("Log level must be one of: ", paste(levels, collapse = ", "))
  }
  .Call(R_log_level, set)
}

#' @useDynLib av R_get_open_handles
get_open_handles <- function(){
  .Call(R_get_open_handles)
}

context("Convert Video")

test_that("Set log level", {
  av_log_level(16)
  expect_equal(av_log_level(), 16)
})

q <- 1
n <- 50
framerate <- 10
width <- 640 * q
height <- 480 * q
png_path <- file.path(tempdir(), "frame%03d.png")
png_files <- sprintf(png_path, 1:n)
wonderland <- system.file('samples/Synapsis-Wonderland.mp3', package='av', mustWork = TRUE)

test_that("convert images into video formats", {
  png(png_path, width = width, height = height, res = 72 * q)
  par(ask = FALSE)
  for(i in 1:n){
    hist(rnorm(100, mean = i), main = paste("histogram with mean", i))
  }
  dev.off()
  for(ext in c("mkv", "mp4", "mov", "flv", "gif")){
    filename <- paste0("test.", ext)
    av::av_encode_video(png_files, filename, framerate = framerate, verbose = FALSE)
    expect_true(file.exists(filename))
    info <- av_media_info(filename)

    expect_equal(info$video$width, width)
    expect_equal(info$video$height, height)
    expect_equal(info$video$codec, switch(ext, gif='gif', flv='flv', 'h264'))
    expect_equal(info$video$framerate, framerate)

    # Gif doesn't have video metadata (duration)
    if(ext != 'gif'){
      expect_equal(info$duration, n / framerate)

      # Test converting video to video
      av_video_convert(filename, 'test2.mp4', verbose = FALSE)
      info <- av_media_info('test2.mp4')
      unlink('test2.mp4')
      expect_equal(info$video$width, width)
      expect_equal(info$video$height, height)
      expect_equal(info$video$framerate, framerate)
      expect_null(info$audio)
    }
    unlink(filename)
  }
})

test_that("audio sampling works", {
  for(ext in c("mkv", "mp4", "mov", "flv")){
    filename <- paste0("test.", ext)
    av::av_encode_video(png_files, filename, framerate = framerate, verbose = FALSE, audio = wonderland)
    expect_true(file.exists(filename))
    info <- av_media_info(filename)

    expect_equal(info$video$width, width)
    expect_equal(info$video$height, height)
    expect_equal(info$video$codec, switch(ext, gif='gif', flv='flv', 'h264'))
    expect_equal(info$video$framerate, framerate)

    expect_equal(info$audio$channels, 2)
    expect_equal(info$audio$sample_rate, 44100)

    # Audio stream may slightly alter the duration, 5% margin
    expect_equal(info$duration, n / framerate, tolerance = 0.05)

    # Test converting video to video
    av_video_convert(filename, 'test2.mp4', verbose = FALSE)
    info <- av_media_info('test2.mp4')
    unlink('test2.mp4')
    expect_equal(info$video$width, width)
    expect_equal(info$video$height, height)
    expect_equal(info$video$framerate, framerate)

    expect_equal(info$audio$channels, 2)
    expect_equal(info$audio$sample_rate, 44100)

    # Audio stream may slightly alter the duration, 5% margin
    expect_equal(info$duration, n / framerate, tolerance =  0.05)

    # Cleanup
    unlink(filename)
  }
})

test_that("fractional framerates work", {
  framerate <- 1/5
  av::av_encode_video(png_files, 'test.mp4', framerate = framerate, verbose = FALSE)
  info <- av_media_info('test.mp4')
  unlink('test.mp4')

  expect_equal(info$video$framerate, framerate)
  expect_equal(info$duration, length(png_files) / framerate)

})

test_that("speed up/down filters", {
  for (x in c(0.1, 0.5, 2, 10)){
    framerate <- 25
    filter <- sprintf("setpts=%s*PTS", as.character(x))
    av::av_encode_video(png_files, 'speed.mp4', framerate = framerate, vfilter = filter, verbose = FALSE)
    info <- av_media_info('speed.mp4')
    unlink('speed.mp4')
    expect_equal(info$video$framerate, (framerate / x))
    expect_equal(info$duration, length(png_files) / (framerate / x))
  }
})

test_that("test error handling", {
  wrongfile <- system.file('DESCRIPTION', package='av')
  file.copy(wrongfile, tmp <- tempfile(fileext = '.png'))
  unlink("video.mp4")
  expect_error(av_encode_video(wrongfile, verbose = FALSE), "input")
  expect_false(file.exists("video.mp4"))
  expect_error(av_encode_video(tmp, verbose = -8), "input")
  expect_false(file.exists("video.mp4"))
  expect_error(av_encode_video(png_files, vfilter = "doesnontexist", verbose = -8), "filter")
  expect_false(file.exists("video.mp4"))

  # Test missing stream
  expect_error(av_encode_video(png_files, audio = png_files[1]), "audio")
  expect_false(file.exists("video.mp4"))
  expect_error(av_encode_video(wonderland), "video")
  expect_false(file.exists("video.mp4"))

  # Test time limit
  setTimeLimit(cpu = 2, transient = TRUE)
  timings <- system.time(expect_error(av_encode_video(rep(png_files, 100), verbose = FALSE), "time"))
  setTimeLimit()
  expect_lt(timings[['sys.self']], 2.5)
  expect_equal(get_open_handles(), 0)
})

#' Encode or Convert Audio / Video
#'
#' Encodes a set of images into a video, using custom container format, codec, fps,
#' [video filters](https://ffmpeg.org/ffmpeg-filters.html#Video-Filters), and audio
#' track. If input contains video files, this effectively combines and converts them
#' to the specified output format.
#'
#' The target container format is automatically determined from the file extension of
#' the output file, for example `mp4`, `mkv`, `mov`, or `flv`. Most systems also
#' support `gif` output, but the compression~quality for gif is quite bad.
#' The [gifski](https://cran.r-project.org/package=gifski) package is better suited
#' for generating animated gif files.
#'
#' It is recommended to use let ffmpeg choose the suitable codec for a given container
#' format. Most video formats default to the `libx264` video codec which has excellent
#' compression and works on all modern [browsers](https://caniuse.com/#search=h264),
#' operating systems, and digital TVs.
#'
#' It is safe to interrupt the encoding process by pressing CTRL+C, or via [setTimeLimit].
#' When the encoding is interrupted, the output stream is properly finalized and all open
#' files and resources are properly closed.
#'
#' @export
#' @aliases av
#' @family av
#' @name encoding
#' @useDynLib av R_encode_video
#' @rdname encoding
#' @param input a vector with image or video files. A video input file is treated
#' as a series of images. All input files should have the same width and height.
#' @param output name of the output file. File extension must correspond to a known
#' container format such as `mp4`, `mkv`, `mov`, or `flv`.
#' @param vfilter a string defining an ffmpeg filter graph. This is the same parameter
#' as the `-vf` argument in the `ffmpeg` command line utility.
#' @param framerate video framerate in frames per seconds. This is the input fps, the
#' output fps may be different if you specify a filter that modifies speed or interpolates
#' frames.
#' @param codec name of the video codec as listed in [av_encoders][av_encoders]. The
#' default is `libx264` for most formats, which usually the best choice.
#' @param audio audio or video input file with sound for the output video
#' @param verbose emit some output and a progress meter counting processed images. Must
#' be `TRUE` or `FALSE` or an integer with a valid [av_log_level].
av_encode_video <- function(input, output = "output.mp4", framerate = 24, vfilter = "null",
                            codec = NULL, audio = NULL, verbose = TRUE){
  stopifnot(length(input) > 0)
  input <- normalizePath(input, mustWork = TRUE)
  stopifnot(length(output) == 1)
  output <- normalizePath(output, mustWork = FALSE)
  stopifnot(file.exists(dirname(output)))
  stopifnot(length(framerate) == 1)
  framerate <- as.numeric(framerate)
  vfilter <- as.character(vfilter)
  codec <- as.character(codec)
  if(length(audio))
    audio <- normalizePath(audio, mustWork = TRUE)
  audio <- as.character(audio)
  if(is.logical(verbose))
    verbose <- ifelse(isTRUE(verbose), 32, 16)
  old_log_level <- av_log_level()
  on.exit(av_log_level(old_log_level), add = TRUE)
  av_log_level(verbose)
  .Call(R_encode_video, input, output, framerate, vfilter, codec, audio)
}

#' @rdname encoding
#' @export
#' @param video input video file with optionally also an audio track
av_video_convert <- function(video, output = "output.mp4", verbose = TRUE){
  info <- av_media_info(video)
  if(nrow(info$video) == 0)
    stop("No suitable input video stream found")
  framerate <- info$video$framerate[1]
  audio <- if(length(info$audio) && nrow(info$audio)) video
  av_encode_video(input = video, audio = audio, output = output,
                  framerate = framerate, verbose = verbose)
}

#' @rdname encoding
#' @export
#' @useDynLib av R_convert_audio
#' @param channels number of output channels. Default `NULL` is to match input
#' @param sample_rate output sampling rate. Default `NULL` is to match input
#' @param format a valid format name from the list of `av_muxers()`. Default
#' `NULL` tries to guess a format from the file extension.
#' @param start_time number greater than 0, seeks in the input file to position.
#' @param total_time approximate number of seconds at which to limit the duration
#' of the output file.
av_audio_convert <- function(audio, output = 'output.mp3', format = NULL,
                             channels = NULL, sample_rate = NULL, start_time = NULL,
                             total_time = NULL, verbose = TRUE){
  stopifnot(length(audio) > 0)
  audio <- normalizePath(audio, mustWork = TRUE)
  stopifnot(length(output) == 1)
  output <- normalizePath(output, mustWork = FALSE)
  stopifnot(file.exists(dirname(output)))
  format <- as.character(format)
  if(length(channels))
    stopifnot(is.numeric(channels))
  if(length(sample_rate))
    stopifnot(is.numeric(sample_rate))
  if(is.logical(verbose))
    verbose <- ifelse(isTRUE(verbose), 32, 16)
  if(length(start_time))
    stopifnot(is.numeric(start_time))
  if(length(total_time))
    stopifnot(is.numeric(total_time))
  old_log_level <- av_log_level()
  on.exit(av_log_level(old_log_level), add = TRUE)
  av_log_level(verbose)
  .Call(R_convert_audio, audio, output, format, channels, sample_rate, start_time, total_time)
}

context("Convert Audio")

wonderland <- system.file('samples/Synapsis-Wonderland.mp3', package='av', mustWork = TRUE)

test_that("Audio can be converted to various formats", {
  tmp_wav <- normalizePath(tempfile(fileext = '.wav'), mustWork = FALSE)
  tmp_mp3 <- normalizePath(tempfile(fileext = '.mp3'), mustWork = FALSE)
  tmp_mp4 <- normalizePath(tempfile(fileext = '.mp4'), mustWork = FALSE)
  tmp_mkv <- normalizePath(tempfile(fileext = '.mkv'), mustWork = FALSE)
  input_info <- av_media_info(wonderland)

  # Convert mp3 to mkv (defaults to libvorbis audio)
  expect_equal(av_audio_convert(wonderland, tmp_mkv, verbose = FALSE), tmp_mkv)
  expect_true(file.exists(tmp_mkv))
  mkv_info <- av_media_info(tmp_mkv)
  expect_equal(input_info$duration, mkv_info$duration, tolerance = 0.1)
  expect_equal(nrow(mkv_info$audio), 1)

  # Convert mp3 to mp4 (defaults to aac audio)
  expect_equal(av_audio_convert(wonderland, tmp_mp4, verbose = FALSE), tmp_mp4)
  expect_true(file.exists(tmp_mp4))
  mp4_info <- av_media_info(tmp_mp4)
  expect_equal(input_info$duration, mp4_info$duration, tolerance = 0.1)
  expect_equal(nrow(mp4_info$audio), 1)

  # Convert mp3 to wav
  expect_equal(av_audio_convert(wonderland, tmp_wav, verbose = FALSE), tmp_wav)
  expect_true(file.exists(tmp_wav))
  wav_info <- av_media_info(tmp_wav)
  expect_equal(input_info$duration, wav_info$duration, tolerance = 0.1)
  expect_equal(nrow(wav_info$audio), 1)

  # Convert wav to mp3
  expect_equal(av_audio_convert(tmp_wav, tmp_mp3, verbose = FALSE), tmp_mp3)
  expect_true(file.exists(tmp_mp3))
  mp3_info <- av_media_info(tmp_mp3)
  expect_equal(input_info$duration, mp3_info$duration, tolerance = 0.1)
  expect_equal(nrow(mp3_info$audio), 1)

})

test_that("Audio truncation works as expected",{
  for(start_time in c(0, 5, 20)){
    tmp_mp3 <- normalizePath(tempfile(fileext = '.mp3'), mustWork = FALSE)
    av_audio_convert(wonderland, tmp_mp3, start_time = start_time, total_time = 10, verbose = FALSE)
    info <- av_media_info(tmp_mp3)
    expect_equal(info$duration, 10, tolerance = 0.05)
  }
})

#' Convert video to images
#'
#' Splits a video file in a set of image files. Default image format is
#' jpeg which has good speed and compression. Use `format = "png"` for
#' losless images.
#'
#' For large input videos you can set fps to sample only a limited number
#' of images per second. This also works with fractions, for example `fps = 0.2`
#' will output one image for every 5 sec of video.
#'
#' @export
#' @param video an input video
#' @param destdir directory where to save the png files
#' @param fps sample rate of images. Use `NULL` to get all images.
#' @param format image format such as `png` or `jpeg`, must be available from `av_encoders()`
av_video_images <- function(video, destdir = tempfile(), format = 'jpg', fps = NULL){
  stopifnot(length(video) == 1)
  vfilter <- ifelse(length(fps), paste0('fps=fps=', fps), 'null')
  framerate <- av_media_info(video)$video$framerate
  dir.create(destdir)
  codec <- switch(format, jpeg = 'mjpeg', jpg = 'mjpeg', format)
  output <- file.path(destdir, paste0('image_%6d.', format))
  av_encode_video(input = video, output = output, framerate = framerate,
                  codec = codec, vfilter = vfilter)
  list.files(destdir, pattern = paste0('image_\\d{6}.', format), full.names = TRUE)
}

#' Video Info
#'
#' Get video info such as width, height, format, duration and framerate.
#' This may also be used for audio input files.
#'
#' @export av_video_info av_media_info
#' @aliases av_video_info av_media_info
#' @name info
#' @param file path to an existing file
#' @useDynLib av R_video_info
#' @family av
av_media_info <- function(file){
  file <- normalizePath(file, mustWork = TRUE)
  out <- .Call(R_video_info, file)
  if(length(out$video))
    out$video <- data.frame(out$video, stringsAsFactors = FALSE)
  if(length(out$audio))
    out$audio <- data.frame(out$audio, stringsAsFactors = FALSE)
  out
}

av_video_info <- av_media_info

#' Demo Video
#'
#' Generates random video for testing purposes.
#'
#' @export
#' @inheritParams encoding
#' @inheritParams capturing
#' @param ... other parameters passed to [av_capture_graphics].
#' @name demo
#' @family av
av_demo <- function(output = "demo.mp4", width = 960, height = 720, framerate = 5, verbose = TRUE, ...){
  wonderland <- system.file('samples/Synapsis-Wonderland.mp3', package='av', mustWork = TRUE)
  info <- av_media_info(wonderland)
  len <- framerate * round(info$duration)
  res <- round(72 * min(width, height) / 480)

  # RColorBrewer::brewer.pal(12,"Set3")
  col <- c("#8DD3C7", "#FFFFB3", "#BEBADA", "#FB8072", "#80B1D3", "#FDB462",
    "#B3DE69", "#FCCDE5", "#D9D9D9", "#BC80BD", "#CCEBC5", "#FFED6F")
  video <- av_capture_graphics(output = output,  audio = wonderland, {
    for(i in seq_len(len)){
      if(as.logical(verbose))
        cat(sprintf("\rGenerating plot %d/%d...", i, len), file = stderr())
      graphics::hist(stats::rnorm(100), col = col, main = sprintf("frame %d/%d", i, len))
    }
    if(as.logical(verbose))
      cat("done!\n", file = stderr())
  }, width = width, height = height, res = res, framerate = framerate, verbose = verbose, ...)
  if(interactive())
    utils::browseURL(video)
  return(video)
}

#' AV Formats
#'
#' List supported filters, codecs and container formats.
#'
#' Encoders and decoders convert between raw video/audio frames and compressed stream
#' data for storage or transfer. However such a compressed data stream by itself does
#' not constitute a valid video format yet. Muxers are needed to interleave one or more
#' audio/video/subtitle streams, along with timestamps, metadata, etc, into a proper
#' file format, such as mp4 or mkv.
#'
#' Conversely, demuxers are needed to read a file format into the separate data streams
#' for subsequent decoding into raw audio/video frames. Most operating systems natively
#' support demuxing and decoding common formats and codecs, needed to play those videos.
#' However for encoding and muxing such videos, ffmpeg must have been configured with
#' specific external libraries for a given codec or format.
#'
#' @export
#' @rdname formats
#' @name formats
#' @family av
av_encoders <- function(){
  list_codecs(TRUE)
}

#' @export
#' @rdname formats
av_decoders <- function(){
  list_codecs(FALSE)
}

#' @useDynLib av R_list_codecs
list_codecs <- function(is_encoder){
  res <- as.list(.Call(R_list_codecs))
  names(res) <- c("type", "encoder", "name", "description","format")
  df <- data.frame(res, stringsAsFactors = FALSE)
  df <- df[df$encoder == is_encoder, c("type", "name", "description", "format")]
  df <- df[order(df$name),]
  row.names(df) <- NULL
  df
}

#' @rdname formats
#' @export
#' @useDynLib av R_list_filters
av_filters <- function(){
  res <-as.list(.Call(R_list_filters))
  names(res) <- c("name", "description")
  df <- data.frame(res, stringsAsFactors = FALSE)
  df <- df[order(df$name),]
  row.names(df) <- NULL
  df
}

#' @rdname formats
#' @export
#' @useDynLib av R_list_muxers
av_muxers <- function(){
  res <-as.list(.Call(R_list_muxers))
  names(res) <- c("name", "mime", "extensions", "audio", "video", "description")
  df <- data.frame(res, stringsAsFactors = FALSE)
  df <- df[order(df$name),]
  row.names(df) <- NULL
  df
}

#' @rdname formats
#' @export
#' @useDynLib av R_list_demuxers
av_demuxers <- function(){
  res <-as.list(.Call(R_list_demuxers))
  names(res) <- c("name", "mime", "extensions", "description")
  df <- data.frame(res, stringsAsFactors = FALSE)
  df <- df[order(df$name),]
  row.names(df) <- NULL
  df
}

context("List Formats")

test_that("Listing formats", {
  encoders <- av_encoders()
  decoders <- av_decoders()
  filters <- av_filters()
  muxers <- av_muxers()
  demuxers <- av_demuxers()
  expect_is(encoders, 'data.frame')
  expect_is(decoders, 'data.frame')
  expect_is(filters, 'data.frame')
  expect_is(muxers, 'data.frame')
  expect_is(demuxers, 'data.frame')
  expect_gt(nrow(encoders), 100)
  expect_gt(nrow(decoders), 400)
  expect_gt(nrow(filters), 300)
  expect_gt(nrow(muxers), 100)
  expect_gt(nrow(demuxers), 100)
  expect_equal(names(encoders), c("type", "name", "description", "format"))
  expect_equal(names(decoders), c("type", "name", "description", "format"))
  expect_equal(names(filters), c("name", "description"))
})

context("List Formats")

test_that("Listing formats", {
  encoders <- av_encoders()
  decoders <- av_decoders()
  filters <- av_filters()
  muxers <- av_muxers()
  demuxers <- av_demuxers()
  expect_is(encoders, 'data.frame')
  expect_is(decoders, 'data.frame')
  expect_is(filters, 'data.frame')
  expect_is(muxers, 'data.frame')
  expect_is(demuxers, 'data.frame')
  expect_gt(nrow(encoders), 100)
  expect_gt(nrow(decoders), 400)
  expect_gt(nrow(filters), 300)
  expect_gt(nrow(muxers), 100)
  expect_gt(nrow(demuxers), 100)
  expect_equal(names(encoders), c("type", "name", "description", "format"))
  expect_equal(names(decoders), c("type", "name", "description", "format"))
  expect_equal(names(filters), c("name", "description"))
})

test_that("Critical encoders", {
  encoders <- av_encoders()
  expect_contains(encoders$name, 'libmp3lame')
  expect_contains(encoders$name, 'libvorbis')
  expect_contains(encoders$name, 'libxvid')
  if(!grepl("(Fedora|Rocky|Red Hat)", osVersion)){
    # libx264 is not supported on RHEL/Fedora libavfilter-free-devel but it is
    # in ffmpeg-devel from rpmfusion.
    expect_contains(encoders$name, 'libx264')
  }
})

#' Read audio binary and frequency data
#'
#' Reads raw audio data from any common audio or video format. Use [read_audio_bin] to
#' get raw PCM audio samples, or [read_audio_fft] to stream-convert directly into
#' frequency domain (spectrum) data using FFmpeg built-in FFT.
#'
#' Currently [read_audio_fft] automatically converts input audio to mono channel such
#' that we get a single matrix. Use the `plot()` method on data returned by [read_audio_fft]
#' to show the spectrogram. The [av_spectrogram_video] generates a video that plays
#' the audio while showing an animated spectrogram with moving status bar, which is
#' very cool.
#'
#' @export
#' @rdname read_audio
#' @family av
#' @useDynLib av R_audio_fft
#' @param audio path to the input sound or video file containing the audio stream
#' @param window vector with weights defining the moving [fft window function][hanning].
#' The length of this vector is the size of the window and hence determines the output
#' frequency range.
#' @param overlap value between 0 and 1 of overlap proportion between moving fft windows
#' @param sample_rate downsample audio to reduce FFT output size. Default keeps sample
#' rate from the input file.
#' @param start_time,end_time position (in seconds) to cut input stream to be processed.
#' @examples # Use a 5 sec fragment
#' wonderland <- system.file('samples/Synapsis-Wonderland.mp3', package='av')
#'
#' # Read initial 5 sec as as frequency spectrum
#' fft_data <- read_audio_fft(wonderland, end_time = 5.0)
#' dim(fft_data)
#'
#' # Plot the spectrogram
#' plot(fft_data)
#'
#' # Show other parameters
#' dim(read_audio_fft(wonderland, end_time = 5.0, hamming(2048)))
#' dim(read_audio_fft(wonderland, end_time = 5.0, hamming(4096)))
read_audio_fft <- function(audio, window = hanning(1024), overlap = 0.75,
                           sample_rate = NULL, start_time = NULL, end_time = NULL){
  audio <- normalizePath(audio, mustWork = TRUE)
  overlap <- as.numeric(overlap)
  sample_rate <- as.integer(sample_rate)
  if(!is.numeric(window) || length(window) < 256)
    stop("Window must have at least length 256")
  window <- as.numeric(window)
  if(!length(overlap) || overlap < 0 || overlap >= 1)
    stop("Overlap must be value between 0 and 1")
  info <- av_media_info(audio)
  start_time <- as.numeric(start_time)
  end_time <- as.numeric(end_time)
  av_log_level(16)
  out <- .Call(R_audio_fft, audio, window, overlap, sample_rate, start_time, end_time)

  # Get the real start/end times
  if(!length(start_time) || start_time < 0)
    start_time = 0
  end_time <- attr(out, 'endtime')
  attr(out, 'endtime') = NULL
  attr(out, 'duration') = end_time - start_time;
  attr(out, 'time') <- seq(start_time, end_time, length.out = ncol(out))
  attr(out, 'frequency') <-  seq(0, info$audio$sample_rate * 0.5, length.out = nrow(out))
  attr(out, 'input') <- as.list(info$audio)
  structure(out, class = 'av_fft')
}

#' @export
#' @rdname read_audio
#' @useDynLib av R_audio_bin
#' @param channels number of output channels, set to 1 to convert to mono sound
read_audio_bin <- function(audio, channels = NULL, sample_rate = NULL, start_time = NULL, end_time = NULL){
  audio <- normalizePath(audio, mustWork = TRUE)
  channels <- as.integer(channels)
  sample_rate <- as.integer(sample_rate)
  start_time <- as.numeric(start_time)
  end_time <- as.numeric(end_time)
  av_log_level(16)
  .Call(R_audio_bin, audio, channels, sample_rate, start_time, end_time)
}

read_audio_bin_old <- function(audio, channels = NULL, sample_rate = NULL, start_time = NULL, total_time = NULL){
  tmp <- tempfile(fileext = '.bin')
  on.exit(unlink(tmp))
  audio <- normalizePath(audio, mustWork = TRUE)
  channels <- as.integer(channels)
  av_audio_convert(audio = audio, output = tmp, format = 's32le', channels = channels,
                   sample_rate = sample_rate, start_time = start_time, total_time = total_time, verbose = FALSE)
  out <- readBin(tmp, integer(), file.info(tmp)$size)
  out[is.na(out)] <- -2147483647L #-(.Machine$integer.max)
  if(!length(channels))
    channels <- av_media_info(audio)$audio$channels
  if(!length(sample_rate))
    sample_rate <- av_media_info(audio)$audio$sample_rate
  structure(out, channels = channels, sample_rate = sample_rate)
}

## Do not remove importfrom! https://github.com/ropensci/av/issues/29
#' @importFrom graphics par image legend abline
#' @export
plot.av_fft <- function(x, y, dark = TRUE, legend = TRUE, keep.par = FALSE, useRaster = TRUE, vline = NULL, ...){
  if(!isTRUE(keep.par)){
    # This will also reset the coordinate scale of the plot
    oldpar <- par(no.readonly = TRUE)
    on.exit(par(oldpar))
  }
  col <- if(isTRUE(dark)){
    par(bg='black', col.axis='white', fg='white', family='mono',
                  font=2, col.lab='white', col.main='white')
    #rev(viridisLite::inferno(12))
    c("#FCFFA4FF", "#F5DB4BFF", "#FCAD12FF", "#F78311FF", "#E65D2FFF", "#CB4149FF",
             "#A92E5EFF", "#85216BFF", "#60136EFF", "#3A0963FF", "#140B35FF", "#000004FF")
  } else {
    # hcl.colors(12, "YlOrRd", rev = TRUE)
    c("#FFFFC8", "#FFF4B7", "#FBE49A", "#F8D074", "#F7BA3C", "#F5A100",
      "#F28400", "#ED6200", "#E13C00", "#C32200", "#A20706", "#7D0025")
  }
  par(mar=c(5, 5, 3, 3), mex=0.6)
  image(attr(x, 'time'), attr(x, 'frequency'), t(x),
                  xlab = 'TIME', ylab = 'FREQUENCY (HZ)', col = col, useRaster = useRaster)
  if(isTRUE(legend)){
    input <- attr(x, 'input')
    label <- sprintf("%d channel, %dHz", input$channels, input$sample_rate)
    legend("topright", legend = label, pch='', xjust = 1, yjust = 1, bty='o', cex = 0.7)
  }
  if(length(vline)){
    abline(v = vline, lwd = 2)
  }
}

context("Read raw audio")

wonderland <- system.file('samples/Synapsis-Wonderland.mp3', package='av')

test_that("Audio FFT", {
  extensions <- c("wav", "mkv", "ac3", "flac")
  av_log_level(16) # muffle some warnings about ac3 vbr
  for(ext in extensions){
    filename <- paste0('wonderland.', ext)
    av_audio_convert(wonderland, filename, verbose = FALSE)
    data <- read_audio_fft(filename, window = hanning(2048))
    expect_equal(dim(data)[1], 1024)
    expect_equal(dim(data)[2], 2584, tol = 0.001)
    unlink(filename)
  }
})

test_that("FFT time cutting",{
  fft_orig <- read_audio_fft(wonderland)
  fft0 <- read_audio_fft(wonderland, start_time = 0, end_time = 10)
  fft1 <- read_audio_fft(wonderland, start_time = 10, end_time = 20)
  fft2 <- read_audio_fft(wonderland, start_time = 10, end_time = 100)

  expect_equal(ncol(fft0), ncol(fft_orig)/3, tol = 0.01)
  expect_equal(ncol(fft1), ncol(fft_orig)/3, tol = 0.01)
  expect_equal(ncol(fft2), ncol(fft_orig)*2/3, tol = 0.01)

})

test_that("Read binary audio", {
  out1 <- read_audio_bin(wonderland)
  out2 <- read_audio_bin_old(wonderland)
  expect_identical(out1, out2)

  out1 <- read_audio_bin(wonderland, channels = 1)
  out2 <- read_audio_bin_old(wonderland, channels = 1)
  expect_identical(out1, out2)

  out1_short <- read_audio_bin(wonderland, start_time = 5)
  out2_short <- read_audio_bin_old(wonderland, start_time = 5)
  expect_identical(out1_short, out2_short)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info.R
\name{info}
\alias{info}
\alias{av_media_info}
\alias{av_video_info}
\title{Video Info}
\usage{
av_media_info(file)
}
\arguments{
\item{file}{path to an existing file}
}
\description{
Get video info such as width, height, format, duration and framerate.
This may also be used for audio input files.
}
\seealso{
Other av: 
\code{\link{capturing}},
\code{\link{demo}()},
\code{\link{encoding}},
\code{\link{formats}},
\code{\link{logging}},
\code{\link{read_audio_fft}()}
}
\concept{av}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logging.R
\name{logging}
\alias{logging}
\alias{av_log_level}
\title{Logging}
\usage{
av_log_level(set = NULL)
}
\arguments{
\item{set}{new \href{https://www.ffmpeg.org/doxygen/4.0/group__lavu__log__constants.html}{log level} value}
}
\description{
Get or set the \href{https://www.ffmpeg.org/doxygen/4.0/group__lavu__log__constants.html}{log level}.
}
\seealso{
Other av: 
\code{\link{capturing}},
\code{\link{demo}()},
\code{\link{encoding}},
\code{\link{formats}},
\code{\link{info}},
\code{\link{read_audio_fft}()}
}
\concept{av}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/images.R
\name{av_video_images}
\alias{av_video_images}
\title{Convert video to images}
\usage{
av_video_images(video, destdir = tempfile(), format = "jpg", fps = NULL)
}
\arguments{
\item{video}{an input video}

\item{destdir}{directory where to save the png files}

\item{format}{image format such as \code{png} or \code{jpeg}, must be available from \code{av_encoders()}}

\item{fps}{sample rate of images. Use \code{NULL} to get all images.}
}
\description{
Splits a video file in a set of image files. Default image format is
jpeg which has good speed and compression. Use \code{format = "png"} for
losless images.
}
\details{
For large input videos you can set fps to sample only a limited number
of images per second. This also works with fractions, for example \code{fps = 0.2}
will output one image for every 5 sec of video.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/winfunc.R
\name{window functions}
\alias{window functions}
\alias{hanning}
\alias{hamming}
\alias{blackman}
\alias{bartlett}
\alias{welch}
\alias{flattop}
\alias{bharris}
\alias{bnuttall}
\alias{sine}
\alias{nuttall}
\alias{bhann}
\alias{lanczos}
\alias{gauss}
\alias{tukey}
\alias{dolph}
\alias{cauchy}
\alias{parzen}
\alias{bohman}
\title{Window functions}
\usage{
hanning(n)

hamming(n)

blackman(n)

bartlett(n)

welch(n)

flattop(n)

bharris(n)

bnuttall(n)

sine(n)

nuttall(n)

bhann(n)

lanczos(n)

gauss(n)

tukey(n)

dolph(n)

cauchy(n)

parzen(n)

bohman(n)
}
\arguments{
\item{n}{size of the window (number of weights to generate)}
}
\description{
Several common \href{https://en.wikipedia.org/wiki/Window_function}{windows function}
generators. The functions return a vector of weights to use in \link{read_audio_fft}.
}
\examples{
# Window functions
plot(hanning(1024), type = 'l', xlab = 'window', ylab = 'weight')
lines(hamming(1024), type = 'l', col = 'red')
lines(bartlett(1024), type = 'l', col = 'blue')
lines(welch(1024), type = 'l', col = 'purple')
lines(flattop(1024), type = 'l', col = 'darkgreen')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capture.R
\name{capturing}
\alias{capturing}
\alias{av_capture_graphics}
\alias{av_spectrogram_video}
\title{Record Video from Graphics Device}
\usage{
av_capture_graphics(
  expr,
  output = "output.mp4",
  width = 720,
  height = 480,
  framerate = 1,
  vfilter = "null",
  audio = NULL,
  verbose = TRUE,
  ...
)

av_spectrogram_video(
  audio,
  output = "output.mp4",
  framerate = 25,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{expr}{an R expression that generates the graphics to capture}

\item{output}{name of the output file. File extension must correspond to a known
container format such as \code{mp4}, \code{mkv}, \code{mov}, or \code{flv}.}

\item{width}{width in pixels of the graphics device}

\item{height}{height in pixels of the graphics device}

\item{framerate}{video framerate in frames per seconds. This is the input fps, the
output fps may be different if you specify a filter that modifies speed or interpolates
frames.}

\item{vfilter}{a string defining an ffmpeg filter graph. This is the same parameter
as the \code{-vf} argument in the \code{ffmpeg} command line utility.}

\item{audio}{path to media file with audio stream}

\item{verbose}{emit some output and a progress meter counting processed images. Must
be \code{TRUE} or \code{FALSE} or an integer with a valid \link{av_log_level}.}

\item{...}{extra graphics parameters passed to \code{\link[=png]{png()}}}
}
\description{
Runs the expression and captures all plots into a video. The \link{av_spectrogram_video}
function is a wrapper that plots data from \link{read_audio_fft} with a moving bar and
background audio.
}
\examples{
\donttest{
library(gapminder)
library(ggplot2)
makeplot <- function(){
  datalist <- split(gapminder, gapminder$year)
  lapply(datalist, function(data){
    p <- ggplot(data, aes(gdpPercap, lifeExp, size = pop, color = continent)) +
      scale_size("population", limits = range(gapminder$pop)) + geom_point() + ylim(20, 90) +
      scale_x_log10(limits = range(gapminder$gdpPercap)) + ggtitle(data$year) + theme_classic()
    print(p)
  })
}

# Play 1 plot per sec, and use an interpolation filter to convert into 10 fps
video_file <- file.path(tempdir(), 'output.mp4')
av_capture_graphics(makeplot(), video_file, 1280, 720, res = 144, vfilter = 'framerate=fps=10')
av::av_media_info(video_file)
# utils::browseURL(video_file)}
}
\seealso{
Other av: 
\code{\link{demo}()},
\code{\link{encoding}},
\code{\link{formats}},
\code{\link{info}},
\code{\link{logging}},
\code{\link{read_audio_fft}()}
}
\concept{av}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fft.R
\name{read_audio_fft}
\alias{read_audio_fft}
\alias{read_audio_bin}
\title{Read audio binary and frequency data}
\usage{
read_audio_fft(
  audio,
  window = hanning(1024),
  overlap = 0.75,
  sample_rate = NULL,
  start_time = NULL,
  end_time = NULL
)

read_audio_bin(
  audio,
  channels = NULL,
  sample_rate = NULL,
  start_time = NULL,
  end_time = NULL
)
}
\arguments{
\item{audio}{path to the input sound or video file containing the audio stream}

\item{window}{vector with weights defining the moving \link[=hanning]{fft window function}.
The length of this vector is the size of the window and hence determines the output
frequency range.}

\item{overlap}{value between 0 and 1 of overlap proportion between moving fft windows}

\item{sample_rate}{downsample audio to reduce FFT output size. Default keeps sample
rate from the input file.}

\item{start_time, end_time}{position (in seconds) to cut input stream to be processed.}

\item{channels}{number of output channels, set to 1 to convert to mono sound}
}
\description{
Reads raw audio data from any common audio or video format. Use \link{read_audio_bin} to
get raw PCM audio samples, or \link{read_audio_fft} to stream-convert directly into
frequency domain (spectrum) data using FFmpeg built-in FFT.
}
\details{
Currently \link{read_audio_fft} automatically converts input audio to mono channel such
that we get a single matrix. Use the \code{plot()} method on data returned by \link{read_audio_fft}
to show the spectrogram. The \link{av_spectrogram_video} generates a video that plays
the audio while showing an animated spectrogram with moving status bar, which is
very cool.
}
\examples{
# Use a 5 sec fragment
wonderland <- system.file('samples/Synapsis-Wonderland.mp3', package='av')

# Read initial 5 sec as as frequency spectrum
fft_data <- read_audio_fft(wonderland, end_time = 5.0)
dim(fft_data)

# Plot the spectrogram
plot(fft_data)

# Show other parameters
dim(read_audio_fft(wonderland, end_time = 5.0, hamming(2048)))
dim(read_audio_fft(wonderland, end_time = 5.0, hamming(4096)))
}
\seealso{
Other av: 
\code{\link{capturing}},
\code{\link{demo}()},
\code{\link{encoding}},
\code{\link{formats}},
\code{\link{info}},
\code{\link{logging}}
}
\concept{av}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formats.R
\name{formats}
\alias{formats}
\alias{av_encoders}
\alias{av_decoders}
\alias{av_filters}
\alias{av_muxers}
\alias{av_demuxers}
\title{AV Formats}
\usage{
av_encoders()

av_decoders()

av_filters()

av_muxers()

av_demuxers()
}
\description{
List supported filters, codecs and container formats.
}
\details{
Encoders and decoders convert between raw video/audio frames and compressed stream
data for storage or transfer. However such a compressed data stream by itself does
not constitute a valid video format yet. Muxers are needed to interleave one or more
audio/video/subtitle streams, along with timestamps, metadata, etc, into a proper
file format, such as mp4 or mkv.

Conversely, demuxers are needed to read a file format into the separate data streams
for subsequent decoding into raw audio/video frames. Most operating systems natively
support demuxing and decoding common formats and codecs, needed to play those videos.
However for encoding and muxing such videos, ffmpeg must have been configured with
specific external libraries for a given codec or format.
}
\seealso{
Other av: 
\code{\link{capturing}},
\code{\link{demo}()},
\code{\link{encoding}},
\code{\link{info}},
\code{\link{logging}},
\code{\link{read_audio_fft}()}
}
\concept{av}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/demo.R
\name{demo}
\alias{demo}
\alias{av_demo}
\title{Demo Video}
\usage{
av_demo(
  output = "demo.mp4",
  width = 960,
  height = 720,
  framerate = 5,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{output}{name of the output file. File extension must correspond to a known
container format such as \code{mp4}, \code{mkv}, \code{mov}, or \code{flv}.}

\item{width}{width in pixels of the graphics device}

\item{height}{height in pixels of the graphics device}

\item{framerate}{video framerate in frames per seconds. This is the input fps, the
output fps may be different if you specify a filter that modifies speed or interpolates
frames.}

\item{verbose}{emit some output and a progress meter counting processed images. Must
be \code{TRUE} or \code{FALSE} or an integer with a valid \link{av_log_level}.}

\item{...}{other parameters passed to \link{av_capture_graphics}.}
}
\description{
Generates random video for testing purposes.
}
\seealso{
Other av: 
\code{\link{capturing}},
\code{\link{encoding}},
\code{\link{formats}},
\code{\link{info}},
\code{\link{logging}},
\code{\link{read_audio_fft}()}
}
\concept{av}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encode.R
\name{encoding}
\alias{encoding}
\alias{av_encode_video}
\alias{av}
\alias{av_video_convert}
\alias{av_audio_convert}
\title{Encode or Convert Audio / Video}
\usage{
av_encode_video(
  input,
  output = "output.mp4",
  framerate = 24,
  vfilter = "null",
  codec = NULL,
  audio = NULL,
  verbose = TRUE
)

av_video_convert(video, output = "output.mp4", verbose = TRUE)

av_audio_convert(
  audio,
  output = "output.mp3",
  format = NULL,
  channels = NULL,
  sample_rate = NULL,
  start_time = NULL,
  total_time = NULL,
  verbose = TRUE
)
}
\arguments{
\item{input}{a vector with image or video files. A video input file is treated
as a series of images. All input files should have the same width and height.}

\item{output}{name of the output file. File extension must correspond to a known
container format such as \code{mp4}, \code{mkv}, \code{mov}, or \code{flv}.}

\item{framerate}{video framerate in frames per seconds. This is the input fps, the
output fps may be different if you specify a filter that modifies speed or interpolates
frames.}

\item{vfilter}{a string defining an ffmpeg filter graph. This is the same parameter
as the \code{-vf} argument in the \code{ffmpeg} command line utility.}

\item{codec}{name of the video codec as listed in \link{av_encoders}. The
default is \code{libx264} for most formats, which usually the best choice.}

\item{audio}{audio or video input file with sound for the output video}

\item{verbose}{emit some output and a progress meter counting processed images. Must
be \code{TRUE} or \code{FALSE} or an integer with a valid \link{av_log_level}.}

\item{video}{input video file with optionally also an audio track}

\item{format}{a valid format name from the list of \code{av_muxers()}. Default
\code{NULL} tries to guess a format from the file extension.}

\item{channels}{number of output channels. Default \code{NULL} is to match input}

\item{sample_rate}{output sampling rate. Default \code{NULL} is to match input}

\item{start_time}{number greater than 0, seeks in the input file to position.}

\item{total_time}{approximate number of seconds at which to limit the duration
of the output file.}
}
\description{
Encodes a set of images into a video, using custom container format, codec, fps,
\href{https://ffmpeg.org/ffmpeg-filters.html#Video-Filters}{video filters}, and audio
track. If input contains video files, this effectively combines and converts them
to the specified output format.
}
\details{
The target container format is automatically determined from the file extension of
the output file, for example \code{mp4}, \code{mkv}, \code{mov}, or \code{flv}. Most systems also
support \code{gif} output, but the compression~quality for gif is quite bad.
The \href{https://cran.r-project.org/package=gifski}{gifski} package is better suited
for generating animated gif files.

It is recommended to use let ffmpeg choose the suitable codec for a given container
format. Most video formats default to the \code{libx264} video codec which has excellent
compression and works on all modern \href{https://caniuse.com/#search=h264}{browsers},
operating systems, and digital TVs.

It is safe to interrupt the encoding process by pressing CTRL+C, or via \link{setTimeLimit}.
When the encoding is interrupted, the output stream is properly finalized and all open
files and resources are properly closed.
}
\seealso{
Other av: 
\code{\link{capturing}},
\code{\link{demo}()},
\code{\link{formats}},
\code{\link{info}},
\code{\link{logging}},
\code{\link{read_audio_fft}()}
}
\concept{av}
